function[U,E] = schrodcenter(n, V, a, b, coeff,coeffb)
%%Gives a numerical solution to the Schrodinger equation. 
%% n: The number of sample nodes
%% V: handle to a potential function defined on the interval (0,1
%% a: The left side of the interval
%% b: The right side of the interval
%% coeff: An array corresponding to the coefficients of an arbitrary
%%     centered finite difference scheme for f''. Since the 
%%     coefficients are symmetrical about j, the array should be of form
%%     [j-n, j-n+1, j-n+2, ... j]
%-----
%coefffb is the forward/backward(- sign) coeffcients
%----

%We begin by creating the Schrodinger matrix H to solve
%                       Hu = Eu
close all

len = length(coeff);
%Preset repeated terms to speed up matrix calculation
dx = (b-a)/ (n+1);
terms = zeros(1, len);
for k = 1:len
    terms(k) = -( coeff(k) / dx^2);
end

xval = zeros(n,1);
vval = zeros(n,1);

%Build the matrix
H = zeros(n,n);
for j = 2:(n-1)
    %first and last row dealt separately!! since we might have nonsymmetric
    %stencils being used
    
    for k = 1:len-1
        %Ensure the front entries do not result in an error
        if (a+j*dx)- k*dx > a
            H(j,j-k) = terms(len-k);
        end
    
        %Ensure the last entries do not result in an error
        if (a+j*dx) + k*dx < b
            H(j,j+k) = terms(len-k);
        end
    end
    
    %The middle term cannot result in an error
    vval(j) = V(a + j*dx);
    H(j,j) = terms(len) + vval(j);    
    xval(j) = (a + j * dx);
end
%From time to time, error in the approximation of dx causes the matrix
%to miss a checkpoint and add in an extra column, prevent this
H = H(1:n,1:n);

%------------------------
%dealing with first and last rows
%------------------------
terms = -coeffb(2:end);
H(1,1) = terms(1)/dx^2 + V(a+1*dx);
H(n,n) = terms(1)/dx^2 + V(a+n*dx);
for i=2:(length(terms))
H(1,i) = terms(i)/dx^2;
H(n,n-(i-1)) = terms(i)/dx^2;
end

%Get the eigenvectors and eigenvalues of the Schrodinger Matrix
%E: The Energy (eigenvalues)
%U: A matrix for which each column is an eigenvector (wavefunction)
[U,E] = eig(H);
E = diag(E); 
[E i] = sort(E);
%Normalize each vector
for m = 1:n
    U(:,m) = U(:,m) / norm(U(:,m));
end
   
% %Take the complex conjugate in order to plot probability density
% P = U.*conj(U);
% hold on
% 
% 
% %Plot the first three Probability vectors
%  plot(xval,P(:,1),'b');
%  plot(xval,P(:,2),'r')
%  plot(xval,P(:,3),'g')
%  legend('q = 1','q = 2','q = 3')
% 
%  %Plot the Potential Function
%  figure
%  plot(xval,vval);
%  
%  figure;
%  plot(xval,U(:,1));

    
    
    
    
    